/**************************************************************************
***    
*** Copyright (c) 1995-2000 Regents of the University of California,
***               Andrew E. Caldwell, Andrew B. Kahng and Igor L. Markov
*** Copyright (c) 2000-2004 Regents of the University of Michigan,
***               Saurabh N. Adya, Jarrod A. Roy and Igor L. Markov
***
***  Contact author(s): abk@cs.ucsd.edu, imarkov@umich.edu
***  Original Affiliation:   UCLA, Computer Science Department,
***                          Los Angeles, CA 90095-1596 USA
***
***  Permission is hereby granted, free of charge, to any person obtaining 
***  a copy of this software and associated documentation files (the
***  "Software"), to deal in the Software without restriction, including
***  without limitation 
***  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
***  and/or sell copies of the Software, and to permit persons to whom the 
***  Software is furnished to do so, subject to the following conditions:
***
***  The above copyright notice and this permission notice shall be included
***  in all copies or substantial portions of the Software.
***
*** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
*** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
*** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*** CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
*** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
*** THE USE OR OTHER DEALINGS IN THE SOFTWARE.
***
***
***************************************************************************/






#ifndef NETS_H
#define NETS_H

#include <vector>
#include <algorithm>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <ABKCommon/sgi_hash_map.h>

#include "Net.h"

namespace parquetfp
{
 
   class Nodes;
   typedef std::vector<Net>::iterator itNet;

   struct eqstr
   {
     bool operator()(const char* s1, const char* s2) const
     {
       return strcmp(s1, s2) == 0;
     }
   };

   class Nets
   {
   private:
      vector<Net> _nets;
      hash_map<const char*, int, hash<const char*>, eqstr> _name2IdxMap;

   public:
      Nets(char* baseName);
      Nets()
         {}

      void clean(void)
         { _nets.clear(); }

      void parseNets(char* fnameNets);

      void parseWts(char* fnameWts);

      void updateNodeInfo(Nodes& nodes);

      itNet netsBegin(void)
         { return _nets.begin(); }

      itNet netsEnd(void)
         { return _nets.end(); }

      Net& getNet(unsigned index)
         { return _nets[index]; }

      void putNewNet(Net& net)
         { _nets.push_back(net); }

      int getNumPins(void);

      unsigned getNumNets(void)
         { return _nets.size(); }

      void initName2IdxMap(void);
      void putName2IdxEntry(const char* netName, int idx);
      int getIdxFrmName(const char* netName);
   };
}

//using namespace parquetfp;

#endif
